@@ -18,6 +18,7 @@ from tqdm import tqdm
 
 # Import matplotlib and set dpi to 300
 import matplotlib as mpl
+from .model_names import supported_model_names
 mpl.rcParams['figure.dpi'] = 300
 
 # Return with the output tensors detached from gpu
@@ -89,13 +90,6 @@ class InverseLinear(torch.nn.Module):
             self.fc = self.fc.to( dtype=dtype, **kwargs )
         return self
 
-opt_model_sizes = [ "125m", "350m", "1.3b", "2.7b", "6.7b", "13b", "30b", "66b" ]
-galactica_model_sizes = [ "125m", "1.3b", "6.7b", "30b", "120b" ]
-tested_models = [
-    *[ f"facebook/opt-{s}"       for s in opt_model_sizes ],
-    *[ f"facebook/galactica-{s}" for s in galactica_model_sizes ],
-]
-
 class Model():
     """ Wrapper Class for Meta OPT model that allows me to do interpretability
     work on it's activations and modify it's parameters as needed. """
@@ -141,7 +135,7 @@ class Model():
 
     # pylint: disable=attribute-defined-outside-init
     def set_repo( self, model_name: str ):
-        if model_name not in tested_models:
+        if model_name not in supported_model_names:
             warnings.warn( f"Model {model_name} not tested." )
 
         self.model_size = model_name.split('-')[-1]
@@ -192,7 +186,7 @@ class Model():
         self.predictor.to( device )
         self.model.to( device )
 
-    def out_stack(self, tensor_list: List[Tensor]):
+    def out_stack(self, tensor_list: List[Tensor]) -> Tensor:
         if self.use_accelerator or self.device != self.output_device:
             tensor_list = [ t.to(self.output_device) for t in tensor_list ]
         return torch.stack( tensor_list )
@@ -425,6 +419,9 @@ class Model():
     # then: W_o * pre_out       -> output          "attn_out"
     # output: attn_out, attn_weights, (k_i, v_i)
 
+    def get_attn_layers(self):
+        return [ l.self_attn for l in self.model.decoder.layers ]
+
     def prepare_attention_mask( self, inpt: Tensor ):
         decoder = self.model.decoder
         input_shape = input.size()[:-1]
@@ -708,15 +705,22 @@ class Model():
     # functions for 'deleting' neurons from the MLP mid layers
     def delete_ff_keys( self, layer_key_map: Tensor ):
         for layer, key_map in enumerate(layer_key_map):
-            # Get state dict
-            ff_1  = self.model.decoder.layers[ layer ].fc1
-            state_dict = ff_1.state_dict()
+            # 2. Delete the weights going into ff key so it never activates
+            ff_in = self.model.decoder.layers[ layer ].fc1
+            ff_in_params     = ff_in.state_dict()
+            ff_in_weights   : Tensor = ff_in_params['weight']
+            ff_in_biases    : Tensor = ff_in_params['bias']
 
-            # set biases to -inf to 'delete' the keys (due to ReLU)
-            for index, delete in enumerate(key_map):
-                if delete:
-                    state_dict['bias'][index] = - torch.inf
-            ff_1.load_state_dict( state_dict )
+            # Delete the weights going into the neuron (v_proj)
+            for row_index, weights_row in enumerate(ff_in_weights):
+                if key_map[row_index]:
+                    ff_in_weights[row_index] = torch.zeros_like(weights_row)
+                    ff_in_biases[row_index]  = torch.zeros_like(ff_in_biases[row_index])
+
+            ff_in_params.update({'weight': ff_in_weights, 'bias': ff_in_biases})
+            ff_in.load_state_dict(ff_in_params)
+
+            return
 
     def delete_ff_keys_from_files( self, files: List[str] ):
         """Delete ff mid layer neurons from list of numpy files
